#include "jstrings.h"
#include "cp932.h"

/* temporary */
#define MAX_EUCJP_CHARLEN 3

static unsigned short ch_count = 0;
static unsigned short len_count = 0;
static unsigned char ch_buf[MAX_BUFLEN] = {0};
static unsigned char ch_pos = 0;
static unsigned long current_offset = 0;
static unsigned char ch_tmpbuf[MAX_CHARLEN] = {0};
static int is_eucjp(const unsigned char ch, unsigned char *);

int strings_eucjp(const unsigned char ch, unsigned long *file_offset) {
	int size;
	if ((file_offset == NULL) || ((size = is_eucjp(ch, &ch_tmpbuf[0])) < 0)) {
		/* ch is NOT one of current charset or EOF (in case of "offset == NULL") */
		if (ch_count >= stOptions.min_len) {
			if (stOptions.show_lang)
				printf("%s:", "EUCJP");
			if (stOptions.show_linenumber)
				printf("%6ld:", current_offset);
			printf(" %s\n", ch_buf);
		}
		memset(ch_buf, 0, MAX_BUFLEN);
		memset(ch_tmpbuf, 0, (MAX_EUCJP_CHARLEN+1));
		ch_count = 0; len_count = 0; ch_pos = 0;
	}
	else if (size == 0) {
		/* we cannot determine if ch is one of current charset or not yet */
		if (ch_count == 0)
			current_offset = *file_offset;
	}
	else { /* size > 0 */
		/* ch is one of current charset */
		if (ch_count == 0)
			current_offset = *file_offset;
		strncpy(&ch_buf[len_count], ch_tmpbuf, size);
		len_count+=size; ch_count++; 
		memset(ch_tmpbuf, 0, (MAX_EUCJP_CHARLEN+1)); ch_pos = 0;
	}

	if (len_count >= (MAX_BUFLEN - MAX_EUCJP_CHARLEN)) {
		return FALSE;
	}
	else
		return TRUE;
}

/* charset depended */

static int is_eucjp_1byte(const unsigned char ch) {
	if ((ch >= 0x20) && (ch <= 0x7e))
		return 1;
	else if (ch == '\t')
		return 1;
	else
		return 0;
}

static int is_eucjp_katakana2(const unsigned char ch) {
	if ((ch >= 0xa1) && (ch <= 0xdf))
		return 1;
	else
		return 0;
}


static int is_eucjp_zenkaku1(const unsigned char ch) {
	if ((ch >= 0xa1) && (ch <= 0xfe))
		return 1;
//	else if ((ch >= 0xf0) && (ch <= 0xf9))
//		return 1;
	else
		return 0;
}

static int is_eucjp_zenkaku2(const unsigned char tailbyte, const unsigned char leadbyte) {
	int i;
	int j;

	if      ((tailbyte >= 0xa1) && (tailbyte <= 0xfe))
		return 1;
	return 0;
	if( tailbyte < 0x40 )
		return 0;
	if( 0x81 <= leadbyte && leadbyte <= 0x84 ){
		i = leadbyte - 0x81;
	}else if( 0x87 <= leadbyte && leadbyte <= 0x9f ){
		i = leadbyte - 0x87 + 4;
	}else if( 0xe0 <= leadbyte && leadbyte <= 0xea ){
		i = leadbyte - 0xe0 + 29;
	}else if( 0xed <= leadbyte && leadbyte <= 0xee ){
		i = leadbyte - 0xed + 40;
	}else if( 0xfa <= leadbyte && leadbyte <= 0xfc ){
		i = leadbyte - 0xfa + 42;
	}else{
		return 0;
	}/* TODO: GAIJI */
	j = i * 12 + ( ( tailbyte - 0x40 ) >> 4 );
	return valid_cp932_table[ j ] & ( 0x8000 >> ( tailbyte & 0xf ) );
}

static int is_eucjp(const unsigned char ch, unsigned char *tmpbuf) {
	if ((ch_pos == 0) && (is_eucjp_1byte(ch))) {
		tmpbuf[0] = ch;
		return 1;
	}
	else if ((ch_pos == 0) && (ch == 0x8e)) {
		tmpbuf[0] = ch;
		ch_pos = 1;
		return 1;
	}
	else if ((ch_pos == 0) && (is_eucjp_zenkaku1(ch))) {
		tmpbuf[0] = ch;
		ch_pos = 1;
		return 0;
	}
	else if ((ch_pos == 1) && (tmpbuf[0] == 0x8e) && (is_eucjp_katakana2(ch))) {
		tmpbuf[1] = ch;
		ch_pos = 2;
		return 2;
	}
	else if ((ch_pos == 1) && (is_eucjp_zenkaku2(ch, tmpbuf[0]))) {
		tmpbuf[1] = ch;
		ch_pos = 2;
		return 2;
	}
	return -1;
}
